@@ -842,8 +842,8 @@ cal_backend_wcap_add_component (CalBackendWCAP *wcap, icalcomponent *icalcomp, g
 		
 		l = g_list_find_custom (ilist, comp, cal_backend_wcap_instance_list_compare);
 		if (l) {
-			if (old_object)
-				*old_object = icalcomponent_as_ical_string (e_cal_component_get_icalcomponent (l->data));
+			if (old_object && !*old_object)
+				*old_object = e_cal_component_get_as_string (l->data);
 			ilist = g_list_remove_link (ilist, l);
 		}
 		ilist = g_list_insert_sorted (ilist, comp, cal_backend_wcap_instance_list_compare);
@@ -863,7 +863,7 @@ cal_backend_wcap_add_component (CalBackendWCAP *wcap, icalcomponent *icalcomp, g
 	/* Track the mangled uid components */
 	if (g_hash_table_lookup_extended (priv->objects, uid, &orig_key, &orig_value)) {
 		ECalComponentId id;
-		char *calobj = g_strdup (icalcomponent_as_ical_string (e_cal_component_get_icalcomponent (orig_value)));
+		char *calobj = e_cal_component_get_as_string (orig_value);
 
 		id.uid = g_strdup (uid);
 		id.rid = NULL;
@@ -882,7 +882,7 @@ cal_backend_wcap_add_component (CalBackendWCAP *wcap, icalcomponent *icalcomp, g
 
 	g_hash_table_insert (priv->objects, g_strdup (uid), comp);
 
-	char *calobj = g_strdup(icalcomponent_as_ical_string (e_cal_component_get_icalcomponent (comp)));
+	char *calobj = e_cal_component_get_as_string (comp);
 	if(calobj) {
 		e_cal_backend_notify_object_created (E_CAL_BACKEND (wcap), calobj);
 	}
@@ -917,7 +917,7 @@ add_toplevel_component (CalBackendWCAP *wcap, icalcomponent *toplevel, gboolean
 }
 
 void
-cal_backend_wcap_remove_component (CalBackendWCAP *wcap, const char *uid, CalObjModType mod)
+cal_backend_wcap_remove_component (CalBackendWCAP *wcap, const char *uid, CalObjModType mod, char **old_object)
 {
 	CalBackendWCAPPrivate *priv = wcap->priv;
 	ECalComponent *comp;
@@ -965,8 +965,10 @@ cal_backend_wcap_remove_component (CalBackendWCAP *wcap, const char *uid, CalObj
 		e_cal_component_get_uid (remove_comp, &mangled_uid);
 		if (g_hash_table_lookup_extended (priv->objects, mangled_uid, &orig_key, &orig_value)) {
 			ECalComponentId id;
-			char *calobj = g_strdup (icalcomponent_as_ical_string (e_cal_component_get_icalcomponent (remove_comp)));
+			char *calobj = e_cal_component_get_as_string (remove_comp);
 
+			if (old_object && !*old_object)
+				*old_object = g_strdup (calobj);
 			id.uid = g_strdup (mangled_uid);
 			id.rid = NULL;
 			e_cal_backend_notify_object_removed (E_CAL_BACKEND(wcap), &id, calobj, NULL);
@@ -1001,7 +1003,7 @@ remove_icalcomp_list (CalBackendWCAP *wcap, icalcomponent *icalcomp)
 			e_cal_component_set_icalcomponent (comp, subcomp);
 			sunone_util_mangle_uid (comp);
 			e_cal_component_get_uid (comp, &mangled_uid);
-			cal_backend_wcap_remove_component (wcap, mangled_uid, CALOBJ_MOD_THIS);
+			cal_backend_wcap_remove_component (wcap, mangled_uid, CALOBJ_MOD_THIS, NULL);
 			e_cal_component_set_icalcomponent (comp, NULL);
 		}
 		subcomp = icalcomponent_get_next_component (icalcomp,
@@ -1705,30 +1707,24 @@ cal_backend_wcap_get_static_capabilities (ECalBackendSync *backend, EDataCal *ca
 }
 
 static ECalBackendSyncStatus 
-go_online (CalBackendWCAP *wcap)
+go_online_with_thread (CalBackendWCAP *wcap, SunOneAccount *account)
 {
 	CalBackendWCAPPrivate *priv = wcap->priv;
 	icalcomponent *icalcomp;
-	SunOneAccount *account;
 	SunOneConnection *cnc;
 	guint error_code;
 	SunOneConnectionPreferences *prefs;
 	char *key;
 	const char *uristr;
+	gboolean readonly;
 
 	g_mutex_lock (priv->open_lock);
 
 	uristr = e_cal_backend_get_uri (E_CAL_BACKEND (wcap));
-	account = sunone_component_get_account_from_uri (global_sunone_component, uristr);
-	if (!account) {
-		g_mutex_unlock (priv->open_lock);
-		priv->open_error = GNOME_Evolution_Calendar_OtherError;
-		return priv->open_error;
-	}
 	cnc = sunone_account_get_connection (account);
 	if (!cnc) {
-		g_object_unref (account);
 		g_mutex_unlock (priv->open_lock);
+		e_cal_backend_notify_readonly (E_CAL_BACKEND(wcap), TRUE);
 		priv->open_error = GNOME_Evolution_Calendar_OtherError;
 		return priv->open_error;
 	}
@@ -1737,8 +1733,8 @@ go_online (CalBackendWCAP *wcap)
 		g_free (priv->calid);
 	priv->calid = sunone_util_get_calid_from_uri (uristr);
 	if (!priv->calid) {
-		g_object_unref (account);
 		g_mutex_unlock (priv->open_lock);
+		e_cal_backend_notify_readonly (E_CAL_BACKEND(wcap), TRUE);
 		priv->open_error = GNOME_Evolution_Calendar_OtherError;
 		return priv->open_error;
 	}
@@ -1749,15 +1745,14 @@ go_online (CalBackendWCAP *wcap)
 		g_free (priv->cache_name);
 	priv->cache_name = get_cache_name (wcap, TRUE);
 	priv->so_cnc = g_object_ref (cnc);
-	read_cache (wcap);
 
 	/* load timezones */
 	error_code = sunone_connection_get_all_timezones (priv->so_cnc, &icalcomp);
 
 	if (!SUNONE_ERROR_IS_SUCCESSFUL (error_code)) {
 		cal_backend_wcap_clean_wcap (wcap, TRUE);
-		g_object_unref (account);
 		g_mutex_unlock (priv->open_lock);
+		e_cal_backend_notify_readonly (E_CAL_BACKEND(wcap), TRUE);
 		priv->open_error = GNOME_Evolution_Calendar_OtherError;
 		return priv->open_error;
 	}
@@ -1772,8 +1767,8 @@ go_online (CalBackendWCAP *wcap)
 	priv->props = sunone_connection_get_calprops (priv->so_cnc, priv->calid, FALSE);
 	if (!priv->props) {
 		cal_backend_wcap_clean_wcap (wcap, TRUE);
-		g_object_unref (account);
 		g_mutex_unlock (priv->open_lock);
+		e_cal_backend_notify_readonly (E_CAL_BACKEND(wcap), TRUE);
 		priv->open_error = GNOME_Evolution_Calendar_OtherError;
 		return priv->open_error;
 	}
@@ -1782,8 +1777,8 @@ go_online (CalBackendWCAP *wcap)
 	prefs = sunone_connection_get_preferences (priv->so_cnc);
 	if (!prefs) {
 		cal_backend_wcap_clean_wcap (wcap, TRUE);
-		g_object_unref (account);
 		g_mutex_unlock (priv->open_lock);
+		e_cal_backend_notify_readonly (E_CAL_BACKEND(wcap), TRUE);
 		priv->open_error = GNOME_Evolution_Calendar_OtherError;
 		return priv->open_error;
 	}
@@ -1816,8 +1811,8 @@ go_online (CalBackendWCAP *wcap)
 								   &icalcomp, &priv->timestamp);
 	if (!SUNONE_ERROR_IS_SUCCESSFUL (error_code)) {
 		cal_backend_wcap_clean_wcap (wcap, TRUE);
-		g_object_unref (account);
 		g_mutex_unlock (priv->open_lock);
+		e_cal_backend_notify_readonly (E_CAL_BACKEND(wcap), TRUE);
 
 		if (error_code == SUNONE_ERROR_ACCESS_DENIED) {
 			priv->open_error = GNOME_Evolution_Calendar_PermissionDenied;
@@ -1844,8 +1839,10 @@ go_online (CalBackendWCAP *wcap)
 	sunone_connection_add_wcap (priv->so_cnc, key, (gpointer)wcap);
 	g_free (key);
 
+	cal_backend_wcap_is_read_only (E_CAL_BACKEND_SYNC(wcap), NULL, &readonly);
+	e_cal_backend_notify_readonly (E_CAL_BACKEND(wcap), readonly);
+
 	priv->timeout_id = g_timeout_add (sunone_account_get_poll_interval (account) * 60 * 1000, cal_backend_wcap_poll_cb, wcap);
-	g_object_unref (account);
 
 	cal_backend_wcap_write_cache (wcap);
 
@@ -1854,6 +1851,28 @@ go_online (CalBackendWCAP *wcap)
 	return GNOME_Evolution_Calendar_Success;
 }
 
+static ECalBackendSyncStatus 
+go_online (CalBackendWCAP *wcap)
+{
+	SunOneAccount *account;
+	ECalBackendSyncStatus ret;
+	const char *uristr;
+
+	uristr = e_cal_backend_get_uri (E_CAL_BACKEND (wcap));
+	account = sunone_component_get_account_from_uri (global_sunone_component, uristr);
+	if (!account) {
+		e_cal_backend_notify_readonly (E_CAL_BACKEND(wcap), TRUE);
+		wcap->priv->open_error = GNOME_Evolution_Calendar_OtherError;
+		return wcap->priv->open_error;
+	}
+	sunone_account_set_connecting (account, TRUE);
+	ret = go_online_with_thread (wcap, account);
+	sunone_account_set_connecting (account, FALSE);
+	g_object_unref (account);
+
+	return ret;
+}
+
 static ECalBackendSyncStatus
 cal_backend_wcap_open (ECalBackendSync *backend, EDataCal *cal, gboolean only_if_exists,
 			const char *username, const char *password)
@@ -1863,6 +1882,7 @@ cal_backend_wcap_open (ECalBackendSync *backend, EDataCal *cal, gboolean only_if
 	icalcomponent_kind kind;
 	GThread *thread = NULL;
 	GError *error = NULL;
+	const char *uristr;
 
 	d(printf("cal_backend_wcap_open(%p, %d)\n", backend, only_if_exists));
 
@@ -1874,20 +1894,20 @@ cal_backend_wcap_open (ECalBackendSync *backend, EDataCal *cal, gboolean only_if
 	if (kind == ICAL_VTODO_COMPONENT)
 		priv->type = TYPE_TODO;
 	else if (kind == ICAL_VEVENT_COMPONENT)
-		 priv->type = TYPE_EVENT;
+		priv->type = TYPE_EVENT;
+
+	uristr = e_cal_backend_get_uri (E_CAL_BACKEND (wcap));
+	priv->calid = sunone_util_get_calid_from_uri (uristr);
+	if (!priv->calid)
+		return GNOME_Evolution_Calendar_NoSuchCal;
+
+	priv->uri = g_strdup (uristr);
+	priv->cache_name = get_cache_name (wcap, TRUE);
 
 	if (priv->mode == CAL_MODE_LOCAL) {
 		ESource *source;
-		const char *uristr;
 		const char *display_contents = NULL;
 
-		uristr = e_cal_backend_get_uri (E_CAL_BACKEND (wcap));
-		priv->calid = sunone_util_get_calid_from_uri (uristr);
-		if (!priv->calid)
-			return GNOME_Evolution_Calendar_NoSuchCal;
-		priv->uri = g_strdup (uristr);
-		priv->cache_name = get_cache_name (wcap, TRUE);
-
 		source = e_cal_backend_get_source (E_CAL_BACKEND (backend));
 		display_contents = e_source_get_property (source, "offline_sync");
 		if (!display_contents || !g_str_equal (display_contents, "1")) {
@@ -1899,6 +1919,7 @@ cal_backend_wcap_open (ECalBackendSync *backend, EDataCal *cal, gboolean only_if
 
 		return GNOME_Evolution_Calendar_Success;
 	}
+	read_cache (wcap);
 
 	thread = g_thread_create ((GThreadFunc)go_online, wcap, FALSE, &error);
 	if (!thread) {
