@@ -300,12 +300,19 @@ append_to_query_attendees (SoupUri *uri, const char *param, GSList *attendee_lis
 			g_string_append_printf (string, "^LANGUAGE=%s", attendee->language);
 
 		/* Email Address */		
-		if (!strncasecmp (attendee->value, "mailto:", 7) && strchr (attendee->value, '@')) {
-			/*if account is in the attendees, change its email to calid*/
-			if(!strcasecmp (email, attendee->value + 7)) {
-				g_string_append_printf (string, "^%s;", calid);
+		if (!strncasecmp (attendee->value, "mailto:", 7)) {
+			if (strchr (attendee->value, '@')) {
+				/* mailto a real email address,
+				 * if account is in the attendees,
+				 * change its email to calid
+				 */
+				if(!strcasecmp (email, attendee->value + 7)) {
+					g_string_append_printf (string, "^%s;", calid);
+				} else
+					g_string_append_printf (string, "^%s;", attendee->value);
 			} else
-				g_string_append_printf (string, "^%s;", attendee->value);
+				/* mailto, but a calid, strip it */
+				g_string_append_printf (string, "^%s;", attendee->value + 7);
 
 		} else
 			g_string_append_printf (string, "^%s;", attendee->value);			
